import { MaybeAccessor } from "@solid-primitives/utils";
import { Accessor, InitializedResource } from "solid-js";
export type ClipboardSetter = (data: string | ClipboardItem[]) => Promise<void>;
export type NewClipboardItem = (type: string, data: string | Blob | PromiseLike<string | Blob>) => ClipboardItem;
export type HighlightModifier = (el: any) => void;
export type Highlighter = (start?: number, end?: number) => HighlightModifier;
export type CopyToClipboardOptions = {
    value?: any;
    setter?: ClipboardSetter;
    highlight?: HighlightModifier;
};
export type ClipboardResourceItem = {
    readonly type: string;
    readonly text: string | undefined;
    readonly blob: Blob;
};
declare module "solid-js" {
    namespace JSX {
        interface Directives {
            copyToClipboard: CopyToClipboardOptions | true;
        }
    }
}
/**
 * Async write to the clipboard
 * @param data - Data to write to the clipboard - either a string or ClipboardItem array.
 */
export declare const writeClipboard: ClipboardSetter;
/**
 * Async read from the clipboard
 * @return Promise of ClipboardItem array
 */
export declare function readClipboard(): Promise<ClipboardItem[]>;
/**
 * @deprecated Use `readClipboard`, `writeClipboard` and `newClipboardItem` instead.
 */
export declare const makeClipboard: () => [writeClipboard: ClipboardSetter, readClipboard: () => Promise<ClipboardItems | undefined>, newClipboardItem: NewClipboardItem];
/**
 * Creates a new reactive primitive for managing the clipboard.
 *
 * @param data - Data signal to write to the clipboard.
 * @param deferInitial - Sets the value of the clipboard from the signal. defaults to false.
 * @return Returns a resource representing the clipboard elements and children.
 *
 * @example
 * const [data, setData] = createSignal('Foo bar');
 * const [ clipboard, read ] = createClipboard(data);
 */
export declare const createClipboard: (data?: Accessor<string | ClipboardItem[]>, deferInitial?: boolean) => [clipboardItems: InitializedResource<ClipboardResourceItem[]>, refetch: VoidFunction, write: ClipboardSetter];
/**
 * A helpful directive that makes it easy to copy data to clipboard directly.
 * Using it on input should capture it's value, on an HTMLElement will use innerHTML.
 *
 * @param el - Element to bind to
 * @param options - Options to supply the directive with:
 * - `value` — Value to override the clipboard with.
 * - `write` — Optional write method to use for the clipboard action.
 * - `highlight` — The highlight modifier to use for the current element.
 *
 * @example
 * ```ts
 * <input type="text" use:copyToClipboard />
 * ```
 */
export declare const copyToClipboard: (el: HTMLElement, options: MaybeAccessor<CopyToClipboardOptions>) => undefined;
/**
 * newClipboardItem is a wrapper method around creating new ClipboardItems.
 *
 * @param type - The MIME type of the item to create
 * @param data - Data to populate the item with
 * @returns Provides a ClipboardItem object
 */
export declare const newClipboardItem: NewClipboardItem;
/**
 * @deprecated Use `newClipboardItem` instead.
 */
export declare const newItem: NewClipboardItem;
/**
 * A modifier that highlights/selects a range on an HTML element.
 *
 * @param start - Starting point to highlight
 * @param end - Ending point to highlight
 * @returns Returns a modifier function.
 */
export declare const element: Highlighter;
/**
 * A modifier that highlights/selects a range on an HTML input element.
 *
 * @param start - Starting point to highlight
 * @param end - Ending point to highlight
 * @returns Returns a modifier function.
 */
export declare const input: Highlighter;
